;; pbsnodes-xl: summarize information about a TORQUE server
;; Copyright (C) 2008 The Center for High Performance Computing at The
;; University of New Mexico

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;; to build the pbsnodes-xl binary, simply load this file with SBCL
;; something like:
;; sbcl --load pbsnodes-xl.lisp

(defpackage :edu.unm.hpc.pbsnodes-xl
  (:nicknames :pbsnodes-xl)
  (:use :cl
        #+sbcl :sb-mop
        #+sbcl :sb-ext
        :cl-user
        ))

#-sbcl
(error "ONLY SBCL IS SUPPORTED AT THIS TIME")

(in-package :pbsnodes-xl)

(require :asdf)
(require :cffi)
(require :s-xml)

;; foreign PBS lib

;; uncomment and adjust for where your library lives
;; (push #p"/path/to/pbs/lib/" cffi:*foreign-library-directories*)

(cffi:define-foreign-library libpbs
    (t 
     (:or "libpbs.so" ;;"libtorque.so"
	  )))
   
(cffi:use-foreign-library libpbs)

;; cffi 
(load "pbs-interface.lisp")
(defvar *conn* ())

;; (use-package :cl-ppcre)

;; (defvar *build* NIL)

(defun node-resources (resource-list)
  (loop for current = resource-list then (cffi:foreign-slot-value  current 'attrl 'next)
     while  (not (cffi:null-pointer-p current))
     as name = (cffi:foreign-slot-value current 'attrl 'name)
     as resource = (cffi:foreign-slot-value current 'attrl 'resource)
     as fullname = (cond
		       ((and name resource)
			(format NIL "~A.~A" name resource))
		       (name
			(format NIL "~A" name))
		       (resource
			(format NIL "???.~A" resource))
		       (t
			(format NIL "???")))
     collect
     (list (intern fullname :keyword)
	   (cffi:foreign-slot-value current 'attrl 'value))))

;; get a list of all nodes
(defun list-nodes ()
  (let ((first (pbs_statnode *conn* "" (cffi-sys:null-pointer) "")))
    (cons
     (intern "Data" :keyword)
     (sort
      (loop for current = first then (cffi:foreign-slot-value  current 'batch_status 'next)
         while (not (cffi:null-pointer-p current))
         collect
	 (cons 
	  (intern "Node" :keyword)
	  (cons
	   (list (intern "name" :keyword)
		 (cffi:foreign-slot-value current 'batch_status 'name))
	   (node-resources 
	    (cffi:foreign-slot-value current 'batch_status 'attribs))
	   )))
      #'string< :key #'car))
    ))

(defun pbsnodes-x ()
  (setf *conn* (pbs_connect (pbs_default)))
  (if (rest sb-ext:*posix-argv*)
      (format t "~A~%" 
	      (s-xml:print-xml-string (list-nodes) :pretty t))
    (format t "~A~%" 
	    (s-xml:print-xml-string (list-nodes))))
  0)


;; make a binary & die
#+build
(save-lisp-and-die 
 "pbsnodes-xl" 
 :toplevel (function pbsnodes-xl::pbsnodes-x)
 :executable t)
